=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-07-23 14:06:37 +0000
+++ bzrlib/commands.py	2005-07-28 18:46:35 +0000
@@ -1420,14 +1420,12 @@
 class cmd_merge_revert(Command):
     """Reverse all changes since the last commit.
 
-    Only versioned files are affected.
-
-    TODO: Store backups of any files that will be reverted, so
-          that the revert can be undone.          
+    Only versioned files are affected.  By default, any files that are changed
+    will be backed up first.  Backup files have a '~' appended to their name.
     """
-    takes_options = ['revision']
+    takes_options = ['revision', 'no-backup']
 
-    def run(self, revision=None):
+    def run(self, revision=None, no_backup=False):
         from bzrlib.merge import merge
         if revision is None:
             revision = [-1]
@@ -1435,7 +1433,8 @@
             raise BzrCommandError('bzr merge-revert --revision takes exactly 1 argument')
         merge(('.', revision[0]), parse_spec('.'),
               check_clean=False,
-              ignore_zero=True)
+              ignore_zero=True,
+              backup_files=not no_backup)
 
 
 class cmd_assert_fail(Command):
@@ -1500,6 +1499,7 @@
     'update':                 None,
     'long':                   None,
     'root':                   str,
+    'no-backup':              None,
     }
 
 SHORT_OPTIONS = {

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-07-26 17:44:20 +0000
+++ bzrlib/merge.py	2005-07-28 18:46:35 +0000
@@ -1,7 +1,6 @@
-from merge_core import merge_flex
+from merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
 from changeset import generate_changeset, ExceptionConflictHandler
 from changeset import Inventory, Diff3Merge
-from merge_core import ApplyMerge3
 from bzrlib import find_branch
 import bzrlib.osutils
 from bzrlib.errors import BzrCommandError
@@ -166,7 +165,7 @@
 
 def merge(other_revision, base_revision,
           check_clean=True, ignore_zero=False,
-          this_dir=None):
+          this_dir=None, backup_files=False):
     """Merge changes into a tree.
 
     base_revision
@@ -202,7 +201,7 @@
             base_revision = ['.', base_revno]
         base_branch, base_tree = get_tree(base_revision, tempdir, "base")
         merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                    ignore_zero=ignore_zero)
+                    ignore_zero=ignore_zero, backup_files=backup_files)
     finally:
         shutil.rmtree(tempdir)
 
@@ -232,12 +231,15 @@
 
 
 def merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                ignore_zero=False, merge_type="diff3"):
+                ignore_zero=False, merge_type="diff3", backup_files=False):
     merge_types = {"merge3": ApplyMerge3, 
                    "diff3": Diff3Merge}
 
     def merge_factory(base_file, other_file):
-        return merge_types[merge_type](base_file, other_file)
+        contents_change = merge_types[merge_type](base_file, other_file)
+        if backup_files:
+            contents_change = BackupBeforeChange(contents_change)
+        return contents_change
 
     this_tree = get_tree((this_branch.base, None), tempdir, "this")[1]
 

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-07-28 13:37:16 +0000
+++ bzrlib/merge_core.py	2005-07-28 18:46:35 +0000
@@ -1,6 +1,7 @@
 import changeset
 from changeset import Inventory, apply_changeset, invert_dict
 import os.path
+from osutils import backup_file
 from merge3 import Merge3
 
 class ApplyMerge3:
@@ -49,6 +50,25 @@
         else:
             conflict_handler.merge_conflict(new_file, filename, base, other)
 
+
+class BackupBeforeChange:
+    """Contents-change wrapper to back up file first"""
+    def __init__(self, contents_change):
+        self.contents_change = contents_change
+ 
+    def __eq__(self, other):
+        if not isinstance(other, BackupBeforeChange):
+            return False
+        return (self.contents_change == other.contents_change)
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def apply(self, filename, conflict_handler, reverse=False):
+        backup_file(filename)
+        self.contents_change.apply(filename, conflict_handler, reverse)
+
+
 class ThreewayInventory(object):
     def __init__(self, this_inventory, base_inventory, other_inventory):
         self.this = this_inventory
@@ -548,8 +568,25 @@
         """Test diff3 merging"""
         self.do_contents_test(changeset.Diff3Merge)
 
+    def test_contents_merge3(self):
+        """Test diff3 merging"""
+        def backup_merge(base_file, other_file):
+            return BackupBeforeChange(ApplyMerge3(base_file, other_file))
+        builder = self.contents_test_success(backup_merge)
+        def backup_exists(file_id):
+            return os.path.exists(builder.this.full_path(file_id)+"~")
+        assert backup_exists("1")
+        assert backup_exists("2")
+        assert not backup_exists("3")
+        builder.cleanup()
+
     def do_contents_test(self, merge_factory):
         """Test merging with specified ContentsChange factory"""
+        builder = self.contents_test_success(merge_factory)
+        builder.cleanup()
+        self.contents_test_conflicts(merge_factory)
+
+    def contents_test_success(self, merge_factory):
         from inspect import isclass
         builder = MergeBuilder()
         builder.add_file("1", "0", "name1", "text1", 0755)
@@ -572,8 +609,9 @@
         assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
         assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
         assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
-        builder.cleanup()
+        return builder
 
+    def contents_test_conflicts(self, merge_factory):
         builder = MergeBuilder()
         builder.add_file("1", "0", "name1", "text1", 0755)
         builder.change_contents("1", other="text4", this="text3")

